
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gtkmm/main.h>
#include <gtkmm/window.h>
#include <gtkmm/stock.h>
#include <gtkmm/box.h>
#include <gtkmm/treeview.h>
#include <gtkmm/treeviewcolumn.h>
#include <gtkmm/liststore.h>
#include <gtkmm/cellrenderertext.h>
#include <gtkmm/cellrendererpixbuf.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/button.h>
#include <stdlib.h>
#include <iostream>
#include <vector>

#include "myrenderer.h"

using namespace std;

class MyList : public Gtk::Window
{
  public:
    MyList ();
    virtual ~MyList () {};

    friend void user_func (
        Gtk::TreeViewColumn *col,
        Gtk::CellRenderer *cell,
        const Glib::RefPtr<Gtk::TreeModel>& model,
        const Gtk::TreeModel::iterator& iter);

  protected:

    virtual void b_clicked ()
    {
        cout << "button clicked!" << endl;
    }
    virtual void on_selection_changed ();

    virtual Glib::RefPtr<Gtk::TreeModel> create_model ();
    static Glib::ustring id_to_symbol (const Gtk::StockID& id);

    class ModelColumns : public Gtk::TreeModel::ColumnRecord
    {
      public:
        Gtk::TreeModelColumn< Glib::RefPtr<Gdk::Pixbuf> > icon;
        Gtk::TreeModelColumn< Glib::ustring >             symbol;
        Gtk::TreeModelColumn< Glib::ustring >             text;
        Gtk::TreeModelColumn< gpointer >                  button;
        Gtk::TreeModelColumn< Gtk::StockID >              id;
        ModelColumns ()
        {
            add (icon);
            add (symbol);
            add (text);
            add (button);
            add (id);
        }
    };
    ModelColumns m_cols;
    Gtk::TreeView m_Tree;
    Glib::RefPtr<Gtk::TreeSelection> m_TreeSelection;
    Glib::RefPtr<Gtk::TreeModel> m_TreeModel;
};

MyList *pMy;

void user_func (
    Gtk::TreeViewColumn *col,
    Gtk::CellRenderer *cell,
    const Glib::RefPtr<Gtk::TreeModel>& model,
    const Gtk::TreeModel::iterator& iter)
{
    const Gtk::TreeModel::Row row = *iter;

    gpointer pBut = row[pMy->m_cols.button];
//    ((Gtk::CellRendererButton *) cell)->gobj ()->button = (gpointer)pBut;
    ((Gtk::CellRendererButton *) cell)->property_button () = pBut;
}

MyList::MyList ()
{
    Gtk::HBox *box = manage (new Gtk::HBox);
    Gtk::ScrolledWindow *pScroll = manage (new Gtk::ScrolledWindow);
    pScroll->set_policy (Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
    m_TreeModel = create_model ();
    m_Tree.set_model (m_TreeModel);
    pScroll->add (m_Tree);
    m_Tree.show ();
    box->pack_start (*pScroll);
    add (*box);
    {
        Gtk::TreeView::Column *pCol =
            Gtk::manage (new Gtk::TreeView::Column ("Symbol"));
        pCol->pack_start (m_cols.icon, false);
        pCol->pack_start (m_cols.symbol);
        m_Tree.append_column (*pCol);
    }
    m_Tree.append_column ("Text", m_cols.text);
    {
        Gtk::CellRendererButton *prend = Gtk::manage (new Gtk::CellRendererButton());

        Gtk::TreeViewColumn::SlotCellData *slot =
            new Gtk::TreeViewColumn::SlotCellData (&user_func);

        Gtk::TreeView::Column *pMyCol =
            Gtk::manage (new Gtk::TreeView::Column ("Button widget",
                                                    *prend));

        pMyCol->set_renderer(*prend, m_cols.button);
        pMyCol->set_cell_data_func (*prend, *slot);

        m_Tree.append_column (*pMyCol);
    }
    m_Tree.append_column ("ID", m_cols.id);

    m_TreeSelection = m_Tree.get_selection ();
    m_TreeSelection->set_mode (Gtk::SELECTION_SINGLE);
    m_TreeSelection->signal_changed().connect(SigC::slot(*this,
        &MyList::on_selection_changed));

    show_all_children ();
}

Glib::RefPtr<Gtk::TreeModel> MyList::create_model ()
{
    Glib::RefPtr<Gtk::ListStore> refStore = Gtk::ListStore::create (m_cols);
    refStore->set_sort_column_id (m_cols.id, Gtk::SORT_ASCENDING);

    typedef std::vector<Gtk::StockID> vec_IDs;
    const vec_IDs vecIDs = Gtk::Stock::get_ids ();
    // iterate through them, populating the ListStore as appropriate

    for(vec_IDs::const_iterator iterIDs = vecIDs.begin();
        iterIDs != vecIDs.end(); ++iterIDs)
    {
        const Gtk::StockID& stockid = *iterIDs;

    // Add a new empty row to the ListStore:
        Gtk::TreeModel::Row row = *(refStore->append());

    // Populate ID and symbol columns:
        row[m_cols.id]     = stockid;
        row[m_cols.symbol] = id_to_symbol(stockid);

    // get a Gtk::StockItem object for use:
        Gtk::StockItem item;
        if(Gtk::Stock::lookup(stockid, item))
        {
      // Populate Accel column, if the stock item has an accelerator key.
            row[m_cols.text] = item.get_label();
        }
        Gtk::Button *pBut = Gtk::manage (new Gtk::Button (item.get_label (),
                                                          true));
//        pBut->signal_clicked().connect(SigC::slot(*this, &MyList::b_clicked));
        row[m_cols.button] = (gpointer)pBut->gobj();
//        pBut->show();

    // Populate icon column:
        row[m_cols.icon] = render_icon(stockid, Gtk::ICON_SIZE_MENU);
    }

    return refStore;
}

Glib::ustring MyList::id_to_symbol(const Gtk::StockID& stockid)
{
  static const char prefix[] = "gtk-";
  const size_t prefix_length = sizeof(prefix) - 1;

  const Glib::ustring id = stockid.get_string();
  Glib::ustring::const_iterator id_pos = id.begin();

  Glib::ustring scope;

  if((id.length() >= prefix_length) && (id.compare(0, prefix_length, prefix) == 0))
  {
    scope = "Gtk::Stock::";
    std::advance(id_pos, prefix_length);
  }

  Glib::ustring symbol;
  std::replace_copy(id_pos, id.end(), std::back_inserter(symbol), gunichar('-'), gunichar('_'));

  return scope + symbol.uppercase();
}

// triggered when TreeView selection changes
void MyList::on_selection_changed()
{
  // put all the nice data stuff in the box on the right

  // Access the model "row" corresponding to the selected TreeView "row".
  if(const Gtk::TreeModel::iterator iter = m_TreeSelection->get_selected())
  {
    const Gtk::TreeModel::Row row = *iter;

    // get the data out of the model
    const Gtk::StockID stockid = row[m_cols.id];
    const Glib::ustring symbol = row[m_cols.symbol];
    const Glib::ustring text  = row[m_cols.text];
    const Glib::RefPtr<Gdk::Pixbuf> icon = row[m_cols.icon];
    gpointer ptr = row[m_cols.button];
    Gtk::Button *pBut = Glib::wrap((GtkButton *)ptr);

    cout << "SELECTED >>> symbol: " << symbol << " text: "
         << text << " button: " << pBut->property_label()
         << " ID: " << stockid.get_string() << endl;
  }
  else
  {
    cout << "NO LINE SELECTED" << endl;
  }
}

int main (int argc, char *argv[])
{
    Gtk::Main kit(&argc, &argv);
    MyList list;
    pMy = &list;

    kit.run (list);

    return EXIT_SUCCESS;
}
